# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from influxdb_client.service._base_service import _BaseService


class VariablesService(_BaseService):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):  # noqa: E501,D401,D403
        """VariablesService - a operation defined in OpenAPI."""
        super().__init__(api_client)

    def delete_variables_id(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_variables_id(variable_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_variables_id_with_http_info(variable_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_variables_id_with_http_info(variable_id, **kwargs)  # noqa: E501
            return data

    def delete_variables_id_with_http_info(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_variables_id_with_http_info(variable_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_variables_id_prepare(variable_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables/{variableID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def delete_variables_id_async(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a variable.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_variables_id_prepare(variable_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables/{variableID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _delete_variables_id_prepare(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['variable_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('delete_variables_id', all_params, local_var_params)
        # verify the required parameter 'variable_id' is set
        if ('variable_id' not in local_var_params or
                local_var_params['variable_id'] is None):
            raise ValueError("Missing the required parameter `variable_id` when calling `delete_variables_id`")  # noqa: E501

        path_params = {}
        if 'variable_id' in local_var_params:
            path_params['variableID'] = local_var_params['variable_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def delete_variables_id_labels_id(self, variable_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a label from a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_variables_id_labels_id(variable_id, label_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str label_id: The label ID to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_variables_id_labels_id_with_http_info(variable_id, label_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_variables_id_labels_id_with_http_info(variable_id, label_id, **kwargs)  # noqa: E501
            return data

    def delete_variables_id_labels_id_with_http_info(self, variable_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a label from a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.delete_variables_id_labels_id_with_http_info(variable_id, label_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str label_id: The label ID to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_variables_id_labels_id_prepare(variable_id, label_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables/{variableID}/labels/{labelID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def delete_variables_id_labels_id_async(self, variable_id, label_id, **kwargs):  # noqa: E501,D401,D403
        """Delete a label from a variable.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str label_id: The label ID to delete. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._delete_variables_id_labels_id_prepare(variable_id, label_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables/{variableID}/labels/{labelID}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _delete_variables_id_labels_id_prepare(self, variable_id, label_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['variable_id', 'label_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('delete_variables_id_labels_id', all_params, local_var_params)
        # verify the required parameter 'variable_id' is set
        if ('variable_id' not in local_var_params or
                local_var_params['variable_id'] is None):
            raise ValueError("Missing the required parameter `variable_id` when calling `delete_variables_id_labels_id`")  # noqa: E501
        # verify the required parameter 'label_id' is set
        if ('label_id' not in local_var_params or
                local_var_params['label_id'] is None):
            raise ValueError("Missing the required parameter `label_id` when calling `delete_variables_id_labels_id`")  # noqa: E501

        path_params = {}
        if 'variable_id' in local_var_params:
            path_params['variableID'] = local_var_params['variable_id']  # noqa: E501
        if 'label_id' in local_var_params:
            path_params['labelID'] = local_var_params['label_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_variables(self, **kwargs):  # noqa: E501,D401,D403
        """List all variables.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_variables(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str zap_trace_span: OpenTracing span context
        :param str org: The name of the organization.
        :param str org_id: The organization ID.
        :return: Variables
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_variables_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_variables_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_variables_with_http_info(self, **kwargs):  # noqa: E501,D401,D403
        """List all variables.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_variables_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str zap_trace_span: OpenTracing span context
        :param str org: The name of the organization.
        :param str org_id: The organization ID.
        :return: Variables
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_variables_prepare(**kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variables',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_variables_async(self, **kwargs):  # noqa: E501,D401,D403
        """List all variables.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str zap_trace_span: OpenTracing span context
        :param str org: The name of the organization.
        :param str org_id: The organization ID.
        :return: Variables
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_variables_prepare(**kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variables',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_variables_prepare(self, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['zap_trace_span', 'org', 'org_id']  # noqa: E501
        self._check_operation_params('get_variables', all_params, local_var_params)

        path_params = {}

        query_params = []
        if 'org' in local_var_params:
            query_params.append(('org', local_var_params['org']))  # noqa: E501
        if 'org_id' in local_var_params:
            query_params.append(('orgID', local_var_params['org_id']))  # noqa: E501

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_variables_id(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_variables_id(variable_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_variables_id_with_http_info(variable_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_variables_id_with_http_info(variable_id, **kwargs)  # noqa: E501
            return data

    def get_variables_id_with_http_info(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_variables_id_with_http_info(variable_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_variables_id_prepare(variable_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables/{variableID}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variable',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_variables_id_async(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """Retrieve a variable.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_variables_id_prepare(variable_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables/{variableID}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variable',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_variables_id_prepare(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['variable_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('get_variables_id', all_params, local_var_params)
        # verify the required parameter 'variable_id' is set
        if ('variable_id' not in local_var_params or
                local_var_params['variable_id'] is None):
            raise ValueError("Missing the required parameter `variable_id` when calling `get_variables_id`")  # noqa: E501

        path_params = {}
        if 'variable_id' in local_var_params:
            path_params['variableID'] = local_var_params['variable_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def get_variables_id_labels(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_variables_id_labels(variable_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_variables_id_labels_with_http_info(variable_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_variables_id_labels_with_http_info(variable_id, **kwargs)  # noqa: E501
            return data

    def get_variables_id_labels_with_http_info(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_variables_id_labels_with_http_info(variable_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_variables_id_labels_prepare(variable_id, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables/{variableID}/labels', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelsResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def get_variables_id_labels_async(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        """List all labels for a variable.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelsResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._get_variables_id_labels_prepare(variable_id, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables/{variableID}/labels', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelsResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _get_variables_id_labels_prepare(self, variable_id, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['variable_id', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('get_variables_id_labels', all_params, local_var_params)
        # verify the required parameter 'variable_id' is set
        if ('variable_id' not in local_var_params or
                local_var_params['variable_id'] is None):
            raise ValueError("Missing the required parameter `variable_id` when calling `get_variables_id_labels`")  # noqa: E501

        path_params = {}
        if 'variable_id' in local_var_params:
            path_params['variableID'] = local_var_params['variable_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def patch_variables_id(self, variable_id, variable, **kwargs):  # noqa: E501,D401,D403
        """Update a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_variables_id(variable_id, variable, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param Variable variable: Variable update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.patch_variables_id_with_http_info(variable_id, variable, **kwargs)  # noqa: E501
        else:
            (data) = self.patch_variables_id_with_http_info(variable_id, variable, **kwargs)  # noqa: E501
            return data

    def patch_variables_id_with_http_info(self, variable_id, variable, **kwargs):  # noqa: E501,D401,D403
        """Update a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.patch_variables_id_with_http_info(variable_id, variable, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param Variable variable: Variable update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._patch_variables_id_prepare(variable_id, variable, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables/{variableID}', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variable',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def patch_variables_id_async(self, variable_id, variable, **kwargs):  # noqa: E501,D401,D403
        """Update a variable.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param Variable variable: Variable update to apply (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._patch_variables_id_prepare(variable_id, variable, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables/{variableID}', 'PATCH',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variable',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _patch_variables_id_prepare(self, variable_id, variable, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['variable_id', 'variable', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('patch_variables_id', all_params, local_var_params)
        # verify the required parameter 'variable_id' is set
        if ('variable_id' not in local_var_params or
                local_var_params['variable_id'] is None):
            raise ValueError("Missing the required parameter `variable_id` when calling `patch_variables_id`")  # noqa: E501
        # verify the required parameter 'variable' is set
        if ('variable' not in local_var_params or
                local_var_params['variable'] is None):
            raise ValueError("Missing the required parameter `variable` when calling `patch_variables_id`")  # noqa: E501

        path_params = {}
        if 'variable_id' in local_var_params:
            path_params['variableID'] = local_var_params['variable_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'variable' in local_var_params:
            body_params = local_var_params['variable']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def post_variables(self, variable, **kwargs):  # noqa: E501,D401,D403
        """Create a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_variables(variable, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Variable variable: Variable to create (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_variables_with_http_info(variable, **kwargs)  # noqa: E501
        else:
            (data) = self.post_variables_with_http_info(variable, **kwargs)  # noqa: E501
            return data

    def post_variables_with_http_info(self, variable, **kwargs):  # noqa: E501,D401,D403
        """Create a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_variables_with_http_info(variable, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Variable variable: Variable to create (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_variables_prepare(variable, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variable',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_variables_async(self, variable, **kwargs):  # noqa: E501,D401,D403
        """Create a variable.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param Variable variable: Variable to create (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_variables_prepare(variable, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variable',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_variables_prepare(self, variable, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['variable', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('post_variables', all_params, local_var_params)
        # verify the required parameter 'variable' is set
        if ('variable' not in local_var_params or
                local_var_params['variable'] is None):
            raise ValueError("Missing the required parameter `variable` when calling `post_variables`")  # noqa: E501

        path_params = {}

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'variable' in local_var_params:
            body_params = local_var_params['variable']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def post_variables_id_labels(self, variable_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_variables_id_labels(variable_id, label_mapping, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_variables_id_labels_with_http_info(variable_id, label_mapping, **kwargs)  # noqa: E501
        else:
            (data) = self.post_variables_id_labels_with_http_info(variable_id, label_mapping, **kwargs)  # noqa: E501
            return data

    def post_variables_id_labels_with_http_info(self, variable_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_variables_id_labels_with_http_info(variable_id, label_mapping, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_variables_id_labels_prepare(variable_id, label_mapping, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables/{variableID}/labels', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_variables_id_labels_async(self, variable_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        """Add a label to a variable.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param LabelMapping label_mapping: Label to add (required)
        :param str zap_trace_span: OpenTracing span context
        :return: LabelResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_variables_id_labels_prepare(variable_id, label_mapping, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables/{variableID}/labels', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='LabelResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_variables_id_labels_prepare(self, variable_id, label_mapping, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['variable_id', 'label_mapping', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('post_variables_id_labels', all_params, local_var_params)
        # verify the required parameter 'variable_id' is set
        if ('variable_id' not in local_var_params or
                local_var_params['variable_id'] is None):
            raise ValueError("Missing the required parameter `variable_id` when calling `post_variables_id_labels`")  # noqa: E501
        # verify the required parameter 'label_mapping' is set
        if ('label_mapping' not in local_var_params or
                local_var_params['label_mapping'] is None):
            raise ValueError("Missing the required parameter `label_mapping` when calling `post_variables_id_labels`")  # noqa: E501

        path_params = {}
        if 'variable_id' in local_var_params:
            path_params['variableID'] = local_var_params['variable_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'label_mapping' in local_var_params:
            body_params = local_var_params['label_mapping']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def put_variables_id(self, variable_id, variable, **kwargs):  # noqa: E501,D401,D403
        """Replace a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_variables_id(variable_id, variable, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param Variable variable: Variable to replace (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.put_variables_id_with_http_info(variable_id, variable, **kwargs)  # noqa: E501
        else:
            (data) = self.put_variables_id_with_http_info(variable_id, variable, **kwargs)  # noqa: E501
            return data

    def put_variables_id_with_http_info(self, variable_id, variable, **kwargs):  # noqa: E501,D401,D403
        """Replace a variable.

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.put_variables_id_with_http_info(variable_id, variable, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param Variable variable: Variable to replace (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._put_variables_id_prepare(variable_id, variable, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/variables/{variableID}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variable',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def put_variables_id_async(self, variable_id, variable, **kwargs):  # noqa: E501,D401,D403
        """Replace a variable.

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str variable_id: The variable ID. (required)
        :param Variable variable: Variable to replace (required)
        :param str zap_trace_span: OpenTracing span context
        :return: Variable
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._put_variables_id_prepare(variable_id, variable, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/variables/{variableID}', 'PUT',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='Variable',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _put_variables_id_prepare(self, variable_id, variable, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['variable_id', 'variable', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('put_variables_id', all_params, local_var_params)
        # verify the required parameter 'variable_id' is set
        if ('variable_id' not in local_var_params or
                local_var_params['variable_id'] is None):
            raise ValueError("Missing the required parameter `variable_id` when calling `put_variables_id`")  # noqa: E501
        # verify the required parameter 'variable' is set
        if ('variable' not in local_var_params or
                local_var_params['variable'] is None):
            raise ValueError("Missing the required parameter `variable` when calling `put_variables_id`")  # noqa: E501

        path_params = {}
        if 'variable_id' in local_var_params:
            path_params['variableID'] = local_var_params['variable_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'variable' in local_var_params:
            body_params = local_var_params['variable']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params



import pickle
import sys

import pytest

from influxdb_client import InfluxDBClient, WriteOptions
from influxdb_client.client.write_api import WriteType
from tests.base_test import current_milli_time, BaseTest


class InfluxDBWriterToPickle:

    def __init__(self):
        self.client = InfluxDBClient(url="http://localhost:8086", token="my-token", org="my-org", debug=False)
        self.write_api = self.client.write_api(
            write_options=WriteOptions(write_type=WriteType.batching, batch_size=50_000, flush_interval=10_000))

    def write(self, record):
        self.write_api.write(bucket="my-bucket", record=record)

    def terminate(self) -> None:
        self.write_api.close()
        self.client.close()


class WriteApiPickle(BaseTest):

    def setUp(self) -> None:
        super().setUp()

    def tearDown(self) -> None:
        super().tearDown()

    @pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7 or higher")
    def test_write_line_protocol(self):
        writer = InfluxDBWriterToPickle()

        pickle_out = open("writer.pickle", "wb")
        pickle.dump(writer, pickle_out)
        pickle_out.close()

        writer = pickle.load(open("writer.pickle", "rb"))

        measurement = "h2o_feet_" + str(current_milli_time())
        writer.write(record=f"{measurement},location=coyote_creek water_level=1.0")
        writer.terminate()

        tables = self.query_api.query(
            f'from(bucket: "my-bucket") |> range(start: 0) |> filter(fn: (r) => r._measurement == "{measurement}")')

        self.assertEqual(len(tables), 1)
        self.assertEqual(len(tables[0].records), 1)
        self.assertEqual(tables[0].records[0].get_measurement(), measurement)
        self.assertEqual(tables[0].records[0].get_value(), 1.0)
        self.assertEqual(tables[0].records[0].get_field(), "water_level")



import pytest

from influxdb_client import PermissionResource, Permission, Authorization, AuthorizationsApi
from influxdb_client.rest import ApiException
from tests.base_test import BaseTest


class AuthorizationsClientTest(BaseTest):

    def setUp(self) -> None:
        super(AuthorizationsClientTest, self).setUp()
        self.user = self.users_api.me()
        self.organization = self.find_my_org()

    def test_createAuthorization(self):
        user_resource = PermissionResource(org_id=self.organization.id, type="users")
        read_users = Permission(action="read", resource=user_resource)

        org_resource = PermissionResource(org_id=self.organization.id, type="orgs")
        write_organizations = Permission(action="write", resource=org_resource)

        permissions = [read_users, write_organizations]
        authorization = self.authorizations_api.create_authorization(self.organization.id, permissions)

        self.log(authorization)

        self.assertIsNotNone(authorization)
        self.assertTrue(len(authorization.token) > 0)
        self.assertEqual(authorization.user_id, self.user.id)
        self.assertEqual(authorization.user, self.user.name)
        self.assertEqual(authorization.org_id, self.organization.id)
        self.assertEqual(authorization.org, self.organization.name)
        self.assertEqual(authorization.status, "active")
        self.assertTrue(len(authorization.permissions) == 2)
        self.assertEqual(authorization.permissions[0].resource.type, "users")
        self.assertEqual(authorization.permissions[0].resource.org_id, self.organization.id)
        self.assertEqual(authorization.permissions[0].action, "read")
        self.assertEqual(authorization.permissions[1].resource.type, "orgs")
        self.assertEqual(authorization.permissions[1].resource.org_id, self.organization.id)
        self.assertEqual(authorization.permissions[1].action, "write")

        self.assertTrue(authorization.links is not None)

        self.assertEqual(authorization.links["self"], "/api/v2/authorizations/" + authorization.id)

        self.assertEqual(authorization.links["user"], "/api/v2/users/" + self.user.id)

    def test_authorizationDescription(self):
        organization = self.my_organization

        resource = PermissionResource(org_id=organization.id, type="sources")
        create_source = Permission(action="write", resource=resource)

        permissions = [create_source]
        authorization = Authorization(org_id=organization.id, permissions=permissions)
        authorization.status = "active"
        authorization.description = "My description!"
        created = self.authorizations_api.create_authorization(authorization=authorization)
        self.assertIsNotNone(created)
        self.assertEqual(created.description, "My description!")

    def test_createAuthorizationTask(self):
        resource = PermissionResource(org_id=self.organization.id, type="tasks")
        create_task = Permission(action="read", resource=resource)

        delete_task = Permission(action="write", resource=resource)

        permissions = [create_task, delete_task]
        authorization = self.authorizations_api.create_authorization(self.organization.id, permissions)

        self.assertTrue(len(authorization.permissions) == 2)

        self.assertEqual(authorization.permissions[0].resource.type, "tasks")
        self.assertEqual(authorization.permissions[0].resource.org_id, self.organization.id)
        self.assertEqual(authorization.permissions[0].action, "read")

        self.assertEqual(authorization.permissions[1].resource.type, "tasks")
        self.assertEqual(authorization.permissions[1].resource.org_id, self.organization.id)
        self.assertEqual(authorization.permissions[1].action, "write")

    def test_createAuthorizationBucket(self):
        organization = self.client.organizations_api().create_organization(self.generate_name("Auth Organization"))
        bucket = self.client.buckets_api().create_bucket(bucket_name=self.generate_name("Auth Bucket"),
                                                         retention_rules=BaseTest.retention_rule(),
                                                         org=self.organization)
        resource = PermissionResource(org_id=organization.id, type="buckets", id=bucket.id)
        create_bucket = Permission(action="read", resource=resource)
        delete_bucket = Permission(action="write", resource=resource)
        permissions = [create_bucket, delete_bucket]

        authorization = self.authorizations_api.create_authorization(organization.id, permissions)

        self.log(authorization)
        self.assertTrue(len(authorization.permissions) == 2)
        self.assertEqual(authorization.permissions[0].resource.id, bucket.id)
        self.assertEqual(authorization.permissions[0].resource.type, "buckets")
        self.assertEqual(authorization.permissions[0].action, "read")
        self.assertEqual(authorization.permissions[1].resource.id, bucket.id)
        self.assertEqual(authorization.permissions[1].resource.type, "buckets")
        self.assertEqual(authorization.permissions[1].action, "write")
        self.delete_test_bucket(bucket)

    def test_findAuthorizationsByID(self):
        with pytest.raises(ApiException) as e:
            self.authorization_api().find_authorization_by_id("020f755c3c082000")
        self.assertEqual(e.value.status, 404)
        self.assertIn("authorization not found", e.value.body)

    def test_findAuthorizations(self):
        authorizations = self.authorization_api().find_authorizations()
        size = len(authorizations)
        self.authorization_api().create_authorization(org_id=self.organization.id,
                                                      permissions=self.new_permissions())

    def test_findAuthorizationsByUser(self):
        size = len(self.authorization_api().find_authorizations_by_user(self.user))
        self.authorization_api().create_authorization(org_id=self.organization.id,
                                                      permissions=self.new_permissions())

        self.assertEqual(len(self.authorization_api().find_authorizations_by_user(user=self.user)),
                         size + 1)

    def test_findAuthorizationsByUserName(self):
        size = len(self.authorization_api().find_authorizations_by_user_name(self.user.name))
        self.authorization_api().create_authorization(org_id=self.organization.id,
                                                      permissions=self.new_permissions())

        self.assertEqual(
            len(self.authorization_api().find_authorizations_by_user_name(user_name=self.user.name)),
            size + 1)

    def test_findAuthorizationsByOrg(self):
        size = len(self.authorization_api().find_authorizations_by_org(self.organization))
        self.authorization_api().create_authorization(org_id=self.organization.id,
                                                      permissions=self.new_permissions())

        self.assertEqual(len(self.authorization_api().find_authorizations_by_org(self.organization)),
                         size + 1)

    def test_findAuthorizationsByOrgNotFound(self):
        authorizations_by_org_id = self.authorization_api().find_authorizations_by_org_id("ffffffffffffffff")
        self.assertEqual(len(authorizations_by_org_id), 0)

    def test_updateAuthorizationStatus(self):
        resource = PermissionResource(org_id=self.organization.id, type="users")
        read_user = Permission(action="read", resource=resource)
        permissions = [read_user]

        authorization = self.authorization_api().create_authorization(org_id=self.my_organization.id,
                                                                      permissions=permissions)
        self.assertEqual(authorization.status, "active")

        authorization.status = "inactive"
        authorization = self.authorization_api().update_authorization(authorization)

        self.assertEqual(authorization.status, "inactive")

        authorization.status = "active"
        authorization = self.authorization_api().update_authorization(authorization)
        self.assertEqual(authorization.status, "active")

    def test_deleteAuthorization(self):
        create_authorization = self.authorization_api().create_authorization(self.organization.id,
                                                                             self.new_permissions())
        self.assertIsNotNone(create_authorization)

        found_authorization = self.authorization_api().find_authorization_by_id(create_authorization.id)
        self.assertIsNotNone(found_authorization)

        self.authorization_api().delete_authorization(create_authorization)

        with pytest.raises(ApiException) as e:
            self.authorization_api().find_authorization_by_id(create_authorization.id)
        self.assertIn("authorization not found", e.value.body)

    def test_clone_authorization(self):
        source = self.authorization_api().create_authorization(self.organization.id, self.new_permissions())
        cloned = self.authorization_api().clone_authorization(source.id)

        self.assertIsNotNone(cloned.token)
        self.assertNotEqual(source.token, cloned.token)
        self.assertNotEqual(source.id, cloned.id)

        self.assertEqual(source.user_id, cloned.user_id)
        self.assertEqual(source.user, cloned.user)
        self.assertEqual(source.org_id, cloned.org_id)
        self.assertEqual(source.org, cloned.org)
        self.assertEqual(source.status, cloned.status)
        self.assertEqual(source.description, cloned.description)

        self.assertTrue(len(cloned.permissions), 1)
        self.assertEqual(cloned.permissions[0].action, "read")
        self.assertEqual(cloned.permissions[0].resource.type, "users")
        self.assertEqual(cloned.permissions[0].resource.org_id, self.organization.id)

    def test_cloneAuthorizationNotFound(self):
        with pytest.raises(ApiException) as e:
            self.authorization_api().find_authorization_by_id("020f755c3c082000")
        self.assertIn("authorization not found", e.value.body)

    def authorization_api(self) -> AuthorizationsApi:
        return self.client.authorizations_api()

    def new_permissions(self):
        resource = PermissionResource(org_id=self.organization.id, type="users")
        permission = Permission(action="read", resource=resource)
        return [permission]



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class TelegrafRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'description': 'str',
        'metadata': 'TelegrafRequestMetadata',
        'config': 'str',
        'org_id': 'str'
    }

    attribute_map = {
        'name': 'name',
        'description': 'description',
        'metadata': 'metadata',
        'config': 'config',
        'org_id': 'orgID'
    }

    def __init__(self, name=None, description=None, metadata=None, config=None, org_id=None):  # noqa: E501,D401,D403
        """TelegrafRequest - a model defined in OpenAPI."""  # noqa: E501
        self._name = None
        self._description = None
        self._metadata = None
        self._config = None
        self._org_id = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if metadata is not None:
            self.metadata = metadata
        if config is not None:
            self.config = config
        if org_id is not None:
            self.org_id = org_id

    @property
    def name(self):
        """Get the name of this TelegrafRequest.

        :return: The name of this TelegrafRequest.
        :rtype: str
        """  # noqa: E501
        return self._name

    @name.setter
    def name(self, name):
        """Set the name of this TelegrafRequest.

        :param name: The name of this TelegrafRequest.
        :type: str
        """  # noqa: E501
        self._name = name

    @property
    def description(self):
        """Get the description of this TelegrafRequest.

        :return: The description of this TelegrafRequest.
        :rtype: str
        """  # noqa: E501
        return self._description

    @description.setter
    def description(self, description):
        """Set the description of this TelegrafRequest.

        :param description: The description of this TelegrafRequest.
        :type: str
        """  # noqa: E501
        self._description = description

    @property
    def metadata(self):
        """Get the metadata of this TelegrafRequest.

        :return: The metadata of this TelegrafRequest.
        :rtype: TelegrafRequestMetadata
        """  # noqa: E501
        return self._metadata

    @metadata.setter
    def metadata(self, metadata):
        """Set the metadata of this TelegrafRequest.

        :param metadata: The metadata of this TelegrafRequest.
        :type: TelegrafRequestMetadata
        """  # noqa: E501
        self._metadata = metadata

    @property
    def config(self):
        """Get the config of this TelegrafRequest.

        :return: The config of this TelegrafRequest.
        :rtype: str
        """  # noqa: E501
        return self._config

    @config.setter
    def config(self, config):
        """Set the config of this TelegrafRequest.

        :param config: The config of this TelegrafRequest.
        :type: str
        """  # noqa: E501
        self._config = config

    @property
    def org_id(self):
        """Get the org_id of this TelegrafRequest.

        :return: The org_id of this TelegrafRequest.
        :rtype: str
        """  # noqa: E501
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Set the org_id of this TelegrafRequest.

        :param org_id: The org_id of this TelegrafRequest.
        :type: str
        """  # noqa: E501
        self._org_id = org_id

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, TelegrafRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from influxdb_client.service._base_service import _BaseService


class RestoreService(_BaseService):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):  # noqa: E501,D401,D403
        """RestoreService - a operation defined in OpenAPI."""
        super().__init__(api_client)

    def post_restore_bucket_id(self, bucket_id, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite storage metadata for a bucket with shard info from a backup..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_bucket_id(bucket_id, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str bucket_id: The bucket ID. (required)
        :param str body: Database info serialized as protobuf. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_type:
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_restore_bucket_id_with_http_info(bucket_id, body, **kwargs)  # noqa: E501
        else:
            (data) = self.post_restore_bucket_id_with_http_info(bucket_id, body, **kwargs)  # noqa: E501
            return data

    def post_restore_bucket_id_with_http_info(self, bucket_id, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite storage metadata for a bucket with shard info from a backup..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_bucket_id_with_http_info(bucket_id, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str bucket_id: The bucket ID. (required)
        :param str body: Database info serialized as protobuf. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_type:
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_bucket_id_prepare(bucket_id, body, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/restore/bucket/{bucketID}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='str',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_restore_bucket_id_async(self, bucket_id, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite storage metadata for a bucket with shard info from a backup..

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str bucket_id: The bucket ID. (required)
        :param str body: Database info serialized as protobuf. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_type:
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_bucket_id_prepare(bucket_id, body, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/restore/bucket/{bucketID}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='str',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_restore_bucket_id_prepare(self, bucket_id, body, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['bucket_id', 'body', 'zap_trace_span', 'content_type']  # noqa: E501
        self._check_operation_params('post_restore_bucket_id', all_params, local_var_params)
        # verify the required parameter 'bucket_id' is set
        if ('bucket_id' not in local_var_params or
                local_var_params['bucket_id'] is None):
            raise ValueError("Missing the required parameter `bucket_id` when calling `post_restore_bucket_id`")  # noqa: E501
        # verify the required parameter 'body' is set
        if ('body' not in local_var_params or
                local_var_params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `post_restore_bucket_id`")  # noqa: E501

        path_params = {}
        if 'bucket_id' in local_var_params:
            path_params['bucketID'] = local_var_params['bucket_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501
        if 'content_type' in local_var_params:
            header_params['Content-Type'] = local_var_params['content_type']  # noqa: E501

        body_params = None
        if 'body' in local_var_params:
            body_params = local_var_params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['text/plain'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def post_restore_bucket_metadata(self, bucket_metadata_manifest, **kwargs):  # noqa: E501,D401,D403
        """Create a new bucket pre-seeded with shard info from a backup..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_bucket_metadata(bucket_metadata_manifest, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param BucketMetadataManifest bucket_metadata_manifest: Metadata manifest for a bucket. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: RestoredBucketMappings
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_restore_bucket_metadata_with_http_info(bucket_metadata_manifest, **kwargs)  # noqa: E501
        else:
            (data) = self.post_restore_bucket_metadata_with_http_info(bucket_metadata_manifest, **kwargs)  # noqa: E501
            return data

    def post_restore_bucket_metadata_with_http_info(self, bucket_metadata_manifest, **kwargs):  # noqa: E501,D401,D403
        """Create a new bucket pre-seeded with shard info from a backup..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_bucket_metadata_with_http_info(bucket_metadata_manifest, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param BucketMetadataManifest bucket_metadata_manifest: Metadata manifest for a bucket. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: RestoredBucketMappings
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_bucket_metadata_prepare(bucket_metadata_manifest, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/restore/bucketMetadata', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='RestoredBucketMappings',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_restore_bucket_metadata_async(self, bucket_metadata_manifest, **kwargs):  # noqa: E501,D401,D403
        """Create a new bucket pre-seeded with shard info from a backup..

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param BucketMetadataManifest bucket_metadata_manifest: Metadata manifest for a bucket. (required)
        :param str zap_trace_span: OpenTracing span context
        :return: RestoredBucketMappings
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_bucket_metadata_prepare(bucket_metadata_manifest, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/restore/bucketMetadata', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='RestoredBucketMappings',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_restore_bucket_metadata_prepare(self, bucket_metadata_manifest, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['bucket_metadata_manifest', 'zap_trace_span']  # noqa: E501
        self._check_operation_params('post_restore_bucket_metadata', all_params, local_var_params)
        # verify the required parameter 'bucket_metadata_manifest' is set
        if ('bucket_metadata_manifest' not in local_var_params or
                local_var_params['bucket_metadata_manifest'] is None):
            raise ValueError("Missing the required parameter `bucket_metadata_manifest` when calling `post_restore_bucket_metadata`")  # noqa: E501

        path_params = {}

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501

        body_params = None
        if 'bucket_metadata_manifest' in local_var_params:
            body_params = local_var_params['bucket_metadata_manifest']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def post_restore_kv(self, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite the embedded KV store on the server with a backed-up snapshot..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_kv(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param file body: Full KV snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: PostRestoreKVResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_restore_kv_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.post_restore_kv_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def post_restore_kv_with_http_info(self, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite the embedded KV store on the server with a backed-up snapshot..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_kv_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param file body: Full KV snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: PostRestoreKVResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_kv_prepare(body, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/restore/kv', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='PostRestoreKVResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_restore_kv_async(self, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite the embedded KV store on the server with a backed-up snapshot..

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param file body: Full KV snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: PostRestoreKVResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_kv_prepare(body, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/restore/kv', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type='PostRestoreKVResponse',  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_restore_kv_prepare(self, body, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['body', 'zap_trace_span', 'content_encoding', 'content_type']  # noqa: E501
        self._check_operation_params('post_restore_kv', all_params, local_var_params)
        # verify the required parameter 'body' is set
        if ('body' not in local_var_params or
                local_var_params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `post_restore_kv`")  # noqa: E501

        path_params = {}

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501
        if 'content_encoding' in local_var_params:
            header_params['Content-Encoding'] = local_var_params['content_encoding']  # noqa: E501
        if 'content_type' in local_var_params:
            header_params['Content-Type'] = local_var_params['content_type']  # noqa: E501

        body_params = None
        if 'body' in local_var_params:
            body_params = local_var_params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['text/plain'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def post_restore_shard_id(self, shard_id, body, **kwargs):  # noqa: E501,D401,D403
        """Restore a TSM snapshot into a shard..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_shard_id(shard_id, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str shard_id: The shard ID. (required)
        :param file body: TSM snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_restore_shard_id_with_http_info(shard_id, body, **kwargs)  # noqa: E501
        else:
            (data) = self.post_restore_shard_id_with_http_info(shard_id, body, **kwargs)  # noqa: E501
            return data

    def post_restore_shard_id_with_http_info(self, shard_id, body, **kwargs):  # noqa: E501,D401,D403
        """Restore a TSM snapshot into a shard..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_shard_id_with_http_info(shard_id, body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str shard_id: The shard ID. (required)
        :param file body: TSM snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_shard_id_prepare(shard_id, body, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/restore/shards/{shardID}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_restore_shard_id_async(self, shard_id, body, **kwargs):  # noqa: E501,D401,D403
        """Restore a TSM snapshot into a shard..

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param str shard_id: The shard ID. (required)
        :param file body: TSM snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_shard_id_prepare(shard_id, body, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/restore/shards/{shardID}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_restore_shard_id_prepare(self, shard_id, body, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['shard_id', 'body', 'zap_trace_span', 'content_encoding', 'content_type']  # noqa: E501
        self._check_operation_params('post_restore_shard_id', all_params, local_var_params)
        # verify the required parameter 'shard_id' is set
        if ('shard_id' not in local_var_params or
                local_var_params['shard_id'] is None):
            raise ValueError("Missing the required parameter `shard_id` when calling `post_restore_shard_id`")  # noqa: E501
        # verify the required parameter 'body' is set
        if ('body' not in local_var_params or
                local_var_params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `post_restore_shard_id`")  # noqa: E501

        path_params = {}
        if 'shard_id' in local_var_params:
            path_params['shardID'] = local_var_params['shard_id']  # noqa: E501

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501
        if 'content_encoding' in local_var_params:
            header_params['Content-Encoding'] = local_var_params['content_encoding']  # noqa: E501
        if 'content_type' in local_var_params:
            header_params['Content-Type'] = local_var_params['content_type']  # noqa: E501

        body_params = None
        if 'body' in local_var_params:
            body_params = local_var_params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['text/plain'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params

    def post_restore_sql(self, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite the embedded SQL store on the server with a backed-up snapshot..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_sql(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param file body: Full SQL snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.post_restore_sql_with_http_info(body, **kwargs)  # noqa: E501
        else:
            (data) = self.post_restore_sql_with_http_info(body, **kwargs)  # noqa: E501
            return data

    def post_restore_sql_with_http_info(self, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite the embedded SQL store on the server with a backed-up snapshot..

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.post_restore_sql_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param file body: Full SQL snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_sql_prepare(body, **kwargs)  # noqa: E501

        return self.api_client.call_api(
            '/api/v2/restore/sql', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    async def post_restore_sql_async(self, body, **kwargs):  # noqa: E501,D401,D403
        """Overwrite the embedded SQL store on the server with a backed-up snapshot..

        This method makes an asynchronous HTTP request.

        :param async_req bool
        :param file body: Full SQL snapshot. (required)
        :param str zap_trace_span: OpenTracing span context
        :param str content_encoding: The value tells InfluxDB what compression is applied to the line protocol in the request payload. To make an API request with a GZIP payload, send `Content-Encoding: gzip` as a request header.
        :param str content_type:
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """  # noqa: E501
        local_var_params, path_params, query_params, header_params, body_params = \
            self._post_restore_sql_prepare(body, **kwargs)  # noqa: E501

        return await self.api_client.call_api(
            '/api/v2/restore/sql', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=[],
            files={},
            response_type=None,  # noqa: E501
            auth_settings=[],
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats={},
            urlopen_kw=kwargs.get('urlopen_kw', None))

    def _post_restore_sql_prepare(self, body, **kwargs):  # noqa: E501,D401,D403
        local_var_params = locals()

        all_params = ['body', 'zap_trace_span', 'content_encoding', 'content_type']  # noqa: E501
        self._check_operation_params('post_restore_sql', all_params, local_var_params)
        # verify the required parameter 'body' is set
        if ('body' not in local_var_params or
                local_var_params['body'] is None):
            raise ValueError("Missing the required parameter `body` when calling `post_restore_sql`")  # noqa: E501

        path_params = {}

        query_params = []

        header_params = {}
        if 'zap_trace_span' in local_var_params:
            header_params['Zap-Trace-Span'] = local_var_params['zap_trace_span']  # noqa: E501
        if 'content_encoding' in local_var_params:
            header_params['Content-Encoding'] = local_var_params['content_encoding']  # noqa: E501
        if 'content_type' in local_var_params:
            header_params['Content-Type'] = local_var_params['content_type']  # noqa: E501

        body_params = None
        if 'body' in local_var_params:
            body_params = local_var_params['body']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['text/plain'])  # noqa: E501

        return local_var_params, path_params, query_params, header_params, body_params



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class RemoteConnection(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'org_id': 'str',
        'description': 'str',
        'remote_url': 'str',
        'remote_org_id': 'str',
        'allow_insecure_tls': 'bool'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'org_id': 'orgID',
        'description': 'description',
        'remote_url': 'remoteURL',
        'remote_org_id': 'remoteOrgID',
        'allow_insecure_tls': 'allowInsecureTLS'
    }

    def __init__(self, id=None, name=None, org_id=None, description=None, remote_url=None, remote_org_id=None, allow_insecure_tls=False):  # noqa: E501,D401,D403
        """RemoteConnection - a model defined in OpenAPI."""  # noqa: E501
        self._id = None
        self._name = None
        self._org_id = None
        self._description = None
        self._remote_url = None
        self._remote_org_id = None
        self._allow_insecure_tls = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.org_id = org_id
        if description is not None:
            self.description = description
        self.remote_url = remote_url
        if remote_org_id is not None:
            self.remote_org_id = remote_org_id
        self.allow_insecure_tls = allow_insecure_tls

    @property
    def id(self):
        """Get the id of this RemoteConnection.

        :return: The id of this RemoteConnection.
        :rtype: str
        """  # noqa: E501
        return self._id

    @id.setter
    def id(self, id):
        """Set the id of this RemoteConnection.

        :param id: The id of this RemoteConnection.
        :type: str
        """  # noqa: E501
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501
        self._id = id

    @property
    def name(self):
        """Get the name of this RemoteConnection.

        :return: The name of this RemoteConnection.
        :rtype: str
        """  # noqa: E501
        return self._name

    @name.setter
    def name(self, name):
        """Set the name of this RemoteConnection.

        :param name: The name of this RemoteConnection.
        :type: str
        """  # noqa: E501
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501
        self._name = name

    @property
    def org_id(self):
        """Get the org_id of this RemoteConnection.

        :return: The org_id of this RemoteConnection.
        :rtype: str
        """  # noqa: E501
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Set the org_id of this RemoteConnection.

        :param org_id: The org_id of this RemoteConnection.
        :type: str
        """  # noqa: E501
        if org_id is None:
            raise ValueError("Invalid value for `org_id`, must not be `None`")  # noqa: E501
        self._org_id = org_id

    @property
    def description(self):
        """Get the description of this RemoteConnection.

        :return: The description of this RemoteConnection.
        :rtype: str
        """  # noqa: E501
        return self._description

    @description.setter
    def description(self, description):
        """Set the description of this RemoteConnection.

        :param description: The description of this RemoteConnection.
        :type: str
        """  # noqa: E501
        self._description = description

    @property
    def remote_url(self):
        """Get the remote_url of this RemoteConnection.

        :return: The remote_url of this RemoteConnection.
        :rtype: str
        """  # noqa: E501
        return self._remote_url

    @remote_url.setter
    def remote_url(self, remote_url):
        """Set the remote_url of this RemoteConnection.

        :param remote_url: The remote_url of this RemoteConnection.
        :type: str
        """  # noqa: E501
        if remote_url is None:
            raise ValueError("Invalid value for `remote_url`, must not be `None`")  # noqa: E501
        self._remote_url = remote_url

    @property
    def remote_org_id(self):
        """Get the remote_org_id of this RemoteConnection.

        :return: The remote_org_id of this RemoteConnection.
        :rtype: str
        """  # noqa: E501
        return self._remote_org_id

    @remote_org_id.setter
    def remote_org_id(self, remote_org_id):
        """Set the remote_org_id of this RemoteConnection.

        :param remote_org_id: The remote_org_id of this RemoteConnection.
        :type: str
        """  # noqa: E501
        self._remote_org_id = remote_org_id

    @property
    def allow_insecure_tls(self):
        """Get the allow_insecure_tls of this RemoteConnection.

        :return: The allow_insecure_tls of this RemoteConnection.
        :rtype: bool
        """  # noqa: E501
        return self._allow_insecure_tls

    @allow_insecure_tls.setter
    def allow_insecure_tls(self, allow_insecure_tls):
        """Set the allow_insecure_tls of this RemoteConnection.

        :param allow_insecure_tls: The allow_insecure_tls of this RemoteConnection.
        :type: bool
        """  # noqa: E501
        if allow_insecure_tls is None:
            raise ValueError("Invalid value for `allow_insecure_tls`, must not be `None`")  # noqa: E501
        self._allow_insecure_tls = allow_insecure_tls

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, RemoteConnection):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



import unittest

from urllib3 import HTTPResponse

from influxdb_client.client.exceptions import InfluxDBError


class TestInfluxDBError(unittest.TestCase):
    def test_response(self):
        response = HTTPResponse()
        self.assertEqual(response, InfluxDBError(response=response).response)

    def test_message(self):

        response = HTTPResponse()
        response.headers.add('X-Platform-Error-Code', 'too many requests 1')
        self.assertEqual("too many requests 1", str(InfluxDBError(response=response)))

        response = HTTPResponse()
        response.headers.add('X-Influx-Error', 'too many requests 2')
        self.assertEqual("too many requests 2", str(InfluxDBError(response=response)))

        response = HTTPResponse()
        response.headers.add('X-InfluxDb-Error', 'too many requests 3')
        self.assertEqual("too many requests 3", str(InfluxDBError(response=response)))

        response = HTTPResponse(body='{"code":"too many requests","message":"org 04014de4ed590000 has exceeded limited_write plan limit"}')
        response.headers.add('X-InfluxDb-Error', 'error 3')
        self.assertEqual("org 04014de4ed590000 has exceeded limited_write plan limit", str(InfluxDBError(response=response)))

        response = HTTPResponse(body='org 04014de4ed590000 has exceeded limited_write plan limit')
        response.headers.add('X-InfluxDb-Error', 'error 3')
        self.assertEqual("org 04014de4ed590000 has exceeded limited_write plan limit", str(InfluxDBError(response=response)))

        response = HTTPResponse(reason='too many requests 4')
        self.assertEqual("too many requests 4", str(InfluxDBError(response=response)))

    def test_message_get_retry_after(self):
        response = HTTPResponse(reason="too many requests")
        response.headers.add('Retry-After', '63')

        influx_db_error = InfluxDBError(response=response)
        self.assertEqual("too many requests", str(influx_db_error))
        self.assertEqual("63", influx_db_error.retry_after)

        influx_db_error = InfluxDBError(response=HTTPResponse(reason="too many requests"))
        self.assertEqual("too many requests", str(influx_db_error))
        self.assertEqual(None, influx_db_error.retry_after)

    def test_no_response(self):
        influx_db_error = InfluxDBError(response=None)
        self.assertEqual("no response", str(influx_db_error))
        self.assertIsNone(influx_db_error.response)
        self.assertIsNone(influx_db_error.retry_after)



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class NotificationRuleBase(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'latest_completed': 'datetime',
        'last_run_status': 'str',
        'last_run_error': 'str',
        'id': 'str',
        'endpoint_id': 'str',
        'org_id': 'str',
        'task_id': 'str',
        'owner_id': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'status': 'TaskStatusType',
        'name': 'str',
        'sleep_until': 'str',
        'every': 'str',
        'offset': 'str',
        'runbook_link': 'str',
        'limit_every': 'int',
        'limit': 'int',
        'tag_rules': 'list[TagRule]',
        'description': 'str',
        'status_rules': 'list[StatusRule]',
        'labels': 'list[Label]',
        'links': 'NotificationRuleBaseLinks'
    }

    attribute_map = {
        'latest_completed': 'latestCompleted',
        'last_run_status': 'lastRunStatus',
        'last_run_error': 'lastRunError',
        'id': 'id',
        'endpoint_id': 'endpointID',
        'org_id': 'orgID',
        'task_id': 'taskID',
        'owner_id': 'ownerID',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'status': 'status',
        'name': 'name',
        'sleep_until': 'sleepUntil',
        'every': 'every',
        'offset': 'offset',
        'runbook_link': 'runbookLink',
        'limit_every': 'limitEvery',
        'limit': 'limit',
        'tag_rules': 'tagRules',
        'description': 'description',
        'status_rules': 'statusRules',
        'labels': 'labels',
        'links': 'links'
    }

    def __init__(self, latest_completed=None, last_run_status=None, last_run_error=None, id=None, endpoint_id=None, org_id=None, task_id=None, owner_id=None, created_at=None, updated_at=None, status=None, name=None, sleep_until=None, every=None, offset=None, runbook_link=None, limit_every=None, limit=None, tag_rules=None, description=None, status_rules=None, labels=None, links=None):  # noqa: E501,D401,D403
        """NotificationRuleBase - a model defined in OpenAPI."""  # noqa: E501
        self._latest_completed = None
        self._last_run_status = None
        self._last_run_error = None
        self._id = None
        self._endpoint_id = None
        self._org_id = None
        self._task_id = None
        self._owner_id = None
        self._created_at = None
        self._updated_at = None
        self._status = None
        self._name = None
        self._sleep_until = None
        self._every = None
        self._offset = None
        self._runbook_link = None
        self._limit_every = None
        self._limit = None
        self._tag_rules = None
        self._description = None
        self._status_rules = None
        self._labels = None
        self._links = None
        self.discriminator = None

        if latest_completed is not None:
            self.latest_completed = latest_completed
        if last_run_status is not None:
            self.last_run_status = last_run_status
        if last_run_error is not None:
            self.last_run_error = last_run_error
        if id is not None:
            self.id = id
        self.endpoint_id = endpoint_id
        self.org_id = org_id
        if task_id is not None:
            self.task_id = task_id
        if owner_id is not None:
            self.owner_id = owner_id
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        self.status = status
        self.name = name
        if sleep_until is not None:
            self.sleep_until = sleep_until
        if every is not None:
            self.every = every
        if offset is not None:
            self.offset = offset
        if runbook_link is not None:
            self.runbook_link = runbook_link
        if limit_every is not None:
            self.limit_every = limit_every
        if limit is not None:
            self.limit = limit
        if tag_rules is not None:
            self.tag_rules = tag_rules
        if description is not None:
            self.description = description
        self.status_rules = status_rules
        if labels is not None:
            self.labels = labels
        if links is not None:
            self.links = links

    @property
    def latest_completed(self):
        """Get the latest_completed of this NotificationRuleBase.

        A timestamp ([RFC3339 date/time format](https://docs.influxdata.com/influxdb/latest/reference/glossary/#rfc3339-timestamp)) of the latest scheduled and completed run.

        :return: The latest_completed of this NotificationRuleBase.
        :rtype: datetime
        """  # noqa: E501
        return self._latest_completed

    @latest_completed.setter
    def latest_completed(self, latest_completed):
        """Set the latest_completed of this NotificationRuleBase.

        A timestamp ([RFC3339 date/time format](https://docs.influxdata.com/influxdb/latest/reference/glossary/#rfc3339-timestamp)) of the latest scheduled and completed run.

        :param latest_completed: The latest_completed of this NotificationRuleBase.
        :type: datetime
        """  # noqa: E501
        self._latest_completed = latest_completed

    @property
    def last_run_status(self):
        """Get the last_run_status of this NotificationRuleBase.

        :return: The last_run_status of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._last_run_status

    @last_run_status.setter
    def last_run_status(self, last_run_status):
        """Set the last_run_status of this NotificationRuleBase.

        :param last_run_status: The last_run_status of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._last_run_status = last_run_status

    @property
    def last_run_error(self):
        """Get the last_run_error of this NotificationRuleBase.

        :return: The last_run_error of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._last_run_error

    @last_run_error.setter
    def last_run_error(self, last_run_error):
        """Set the last_run_error of this NotificationRuleBase.

        :param last_run_error: The last_run_error of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._last_run_error = last_run_error

    @property
    def id(self):
        """Get the id of this NotificationRuleBase.

        :return: The id of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._id

    @id.setter
    def id(self, id):
        """Set the id of this NotificationRuleBase.

        :param id: The id of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._id = id

    @property
    def endpoint_id(self):
        """Get the endpoint_id of this NotificationRuleBase.

        :return: The endpoint_id of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._endpoint_id

    @endpoint_id.setter
    def endpoint_id(self, endpoint_id):
        """Set the endpoint_id of this NotificationRuleBase.

        :param endpoint_id: The endpoint_id of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        if endpoint_id is None:
            raise ValueError("Invalid value for `endpoint_id`, must not be `None`")  # noqa: E501
        self._endpoint_id = endpoint_id

    @property
    def org_id(self):
        """Get the org_id of this NotificationRuleBase.

        The ID of the organization that owns this notification rule.

        :return: The org_id of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Set the org_id of this NotificationRuleBase.

        The ID of the organization that owns this notification rule.

        :param org_id: The org_id of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        if org_id is None:
            raise ValueError("Invalid value for `org_id`, must not be `None`")  # noqa: E501
        self._org_id = org_id

    @property
    def task_id(self):
        """Get the task_id of this NotificationRuleBase.

        The ID of the task associated with this notification rule.

        :return: The task_id of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._task_id

    @task_id.setter
    def task_id(self, task_id):
        """Set the task_id of this NotificationRuleBase.

        The ID of the task associated with this notification rule.

        :param task_id: The task_id of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._task_id = task_id

    @property
    def owner_id(self):
        """Get the owner_id of this NotificationRuleBase.

        The ID of creator used to create this notification rule.

        :return: The owner_id of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._owner_id

    @owner_id.setter
    def owner_id(self, owner_id):
        """Set the owner_id of this NotificationRuleBase.

        The ID of creator used to create this notification rule.

        :param owner_id: The owner_id of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._owner_id = owner_id

    @property
    def created_at(self):
        """Get the created_at of this NotificationRuleBase.

        :return: The created_at of this NotificationRuleBase.
        :rtype: datetime
        """  # noqa: E501
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Set the created_at of this NotificationRuleBase.

        :param created_at: The created_at of this NotificationRuleBase.
        :type: datetime
        """  # noqa: E501
        self._created_at = created_at

    @property
    def updated_at(self):
        """Get the updated_at of this NotificationRuleBase.

        :return: The updated_at of this NotificationRuleBase.
        :rtype: datetime
        """  # noqa: E501
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Set the updated_at of this NotificationRuleBase.

        :param updated_at: The updated_at of this NotificationRuleBase.
        :type: datetime
        """  # noqa: E501
        self._updated_at = updated_at

    @property
    def status(self):
        """Get the status of this NotificationRuleBase.

        :return: The status of this NotificationRuleBase.
        :rtype: TaskStatusType
        """  # noqa: E501
        return self._status

    @status.setter
    def status(self, status):
        """Set the status of this NotificationRuleBase.

        :param status: The status of this NotificationRuleBase.
        :type: TaskStatusType
        """  # noqa: E501
        if status is None:
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501
        self._status = status

    @property
    def name(self):
        """Get the name of this NotificationRuleBase.

        Human-readable name describing the notification rule.

        :return: The name of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._name

    @name.setter
    def name(self, name):
        """Set the name of this NotificationRuleBase.

        Human-readable name describing the notification rule.

        :param name: The name of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501
        self._name = name

    @property
    def sleep_until(self):
        """Get the sleep_until of this NotificationRuleBase.

        :return: The sleep_until of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._sleep_until

    @sleep_until.setter
    def sleep_until(self, sleep_until):
        """Set the sleep_until of this NotificationRuleBase.

        :param sleep_until: The sleep_until of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._sleep_until = sleep_until

    @property
    def every(self):
        """Get the every of this NotificationRuleBase.

        The notification repetition interval.

        :return: The every of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._every

    @every.setter
    def every(self, every):
        """Set the every of this NotificationRuleBase.

        The notification repetition interval.

        :param every: The every of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._every = every

    @property
    def offset(self):
        """Get the offset of this NotificationRuleBase.

        Duration to delay after the schedule, before executing check.

        :return: The offset of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._offset

    @offset.setter
    def offset(self, offset):
        """Set the offset of this NotificationRuleBase.

        Duration to delay after the schedule, before executing check.

        :param offset: The offset of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._offset = offset

    @property
    def runbook_link(self):
        """Get the runbook_link of this NotificationRuleBase.

        :return: The runbook_link of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._runbook_link

    @runbook_link.setter
    def runbook_link(self, runbook_link):
        """Set the runbook_link of this NotificationRuleBase.

        :param runbook_link: The runbook_link of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._runbook_link = runbook_link

    @property
    def limit_every(self):
        """Get the limit_every of this NotificationRuleBase.

        Don't notify me more than <limit> times every <limitEvery> seconds. If set, limit cannot be empty.

        :return: The limit_every of this NotificationRuleBase.
        :rtype: int
        """  # noqa: E501
        return self._limit_every

    @limit_every.setter
    def limit_every(self, limit_every):
        """Set the limit_every of this NotificationRuleBase.

        Don't notify me more than <limit> times every <limitEvery> seconds. If set, limit cannot be empty.

        :param limit_every: The limit_every of this NotificationRuleBase.
        :type: int
        """  # noqa: E501
        self._limit_every = limit_every

    @property
    def limit(self):
        """Get the limit of this NotificationRuleBase.

        Don't notify me more than <limit> times every <limitEvery> seconds. If set, limitEvery cannot be empty.

        :return: The limit of this NotificationRuleBase.
        :rtype: int
        """  # noqa: E501
        return self._limit

    @limit.setter
    def limit(self, limit):
        """Set the limit of this NotificationRuleBase.

        Don't notify me more than <limit> times every <limitEvery> seconds. If set, limitEvery cannot be empty.

        :param limit: The limit of this NotificationRuleBase.
        :type: int
        """  # noqa: E501
        self._limit = limit

    @property
    def tag_rules(self):
        """Get the tag_rules of this NotificationRuleBase.

        List of tag rules the notification rule attempts to match.

        :return: The tag_rules of this NotificationRuleBase.
        :rtype: list[TagRule]
        """  # noqa: E501
        return self._tag_rules

    @tag_rules.setter
    def tag_rules(self, tag_rules):
        """Set the tag_rules of this NotificationRuleBase.

        List of tag rules the notification rule attempts to match.

        :param tag_rules: The tag_rules of this NotificationRuleBase.
        :type: list[TagRule]
        """  # noqa: E501
        self._tag_rules = tag_rules

    @property
    def description(self):
        """Get the description of this NotificationRuleBase.

        An optional description of the notification rule.

        :return: The description of this NotificationRuleBase.
        :rtype: str
        """  # noqa: E501
        return self._description

    @description.setter
    def description(self, description):
        """Set the description of this NotificationRuleBase.

        An optional description of the notification rule.

        :param description: The description of this NotificationRuleBase.
        :type: str
        """  # noqa: E501
        self._description = description

    @property
    def status_rules(self):
        """Get the status_rules of this NotificationRuleBase.

        List of status rules the notification rule attempts to match.

        :return: The status_rules of this NotificationRuleBase.
        :rtype: list[StatusRule]
        """  # noqa: E501
        return self._status_rules

    @status_rules.setter
    def status_rules(self, status_rules):
        """Set the status_rules of this NotificationRuleBase.

        List of status rules the notification rule attempts to match.

        :param status_rules: The status_rules of this NotificationRuleBase.
        :type: list[StatusRule]
        """  # noqa: E501
        if status_rules is None:
            raise ValueError("Invalid value for `status_rules`, must not be `None`")  # noqa: E501
        self._status_rules = status_rules

    @property
    def labels(self):
        """Get the labels of this NotificationRuleBase.

        :return: The labels of this NotificationRuleBase.
        :rtype: list[Label]
        """  # noqa: E501
        return self._labels

    @labels.setter
    def labels(self, labels):
        """Set the labels of this NotificationRuleBase.

        :param labels: The labels of this NotificationRuleBase.
        :type: list[Label]
        """  # noqa: E501
        self._labels = labels

    @property
    def links(self):
        """Get the links of this NotificationRuleBase.

        :return: The links of this NotificationRuleBase.
        :rtype: NotificationRuleBaseLinks
        """  # noqa: E501
        return self._links

    @links.setter
    def links(self, links):
        """Set the links of this NotificationRuleBase.

        :param links: The links of this NotificationRuleBase.
        :type: NotificationRuleBaseLinks
        """  # noqa: E501
        self._links = links

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, NotificationRuleBase):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



# coding: utf-8

"""
InfluxDB OSS API Service.

The InfluxDB v2 API provides a programmatic interface for all interactions with InfluxDB. Access the InfluxDB API using the `/api/v2/` endpoint.   # noqa: E501

OpenAPI spec version: 2.0.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401


class StackResources(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'api_version': 'str',
        'resource_id': 'str',
        'kind': 'TemplateKind',
        'template_meta_name': 'str',
        'associations': 'list[StackAssociations]',
        'links': 'StackLinks'
    }

    attribute_map = {
        'api_version': 'apiVersion',
        'resource_id': 'resourceID',
        'kind': 'kind',
        'template_meta_name': 'templateMetaName',
        'associations': 'associations',
        'links': 'links'
    }

    def __init__(self, api_version=None, resource_id=None, kind=None, template_meta_name=None, associations=None, links=None):  # noqa: E501,D401,D403
        """StackResources - a model defined in OpenAPI."""  # noqa: E501
        self._api_version = None
        self._resource_id = None
        self._kind = None
        self._template_meta_name = None
        self._associations = None
        self._links = None
        self.discriminator = None

        if api_version is not None:
            self.api_version = api_version
        if resource_id is not None:
            self.resource_id = resource_id
        if kind is not None:
            self.kind = kind
        if template_meta_name is not None:
            self.template_meta_name = template_meta_name
        if associations is not None:
            self.associations = associations
        if links is not None:
            self.links = links

    @property
    def api_version(self):
        """Get the api_version of this StackResources.

        :return: The api_version of this StackResources.
        :rtype: str
        """  # noqa: E501
        return self._api_version

    @api_version.setter
    def api_version(self, api_version):
        """Set the api_version of this StackResources.

        :param api_version: The api_version of this StackResources.
        :type: str
        """  # noqa: E501
        self._api_version = api_version

    @property
    def resource_id(self):
        """Get the resource_id of this StackResources.

        :return: The resource_id of this StackResources.
        :rtype: str
        """  # noqa: E501
        return self._resource_id

    @resource_id.setter
    def resource_id(self, resource_id):
        """Set the resource_id of this StackResources.

        :param resource_id: The resource_id of this StackResources.
        :type: str
        """  # noqa: E501
        self._resource_id = resource_id

    @property
    def kind(self):
        """Get the kind of this StackResources.

        :return: The kind of this StackResources.
        :rtype: TemplateKind
        """  # noqa: E501
        return self._kind

    @kind.setter
    def kind(self, kind):
        """Set the kind of this StackResources.

        :param kind: The kind of this StackResources.
        :type: TemplateKind
        """  # noqa: E501
        self._kind = kind

    @property
    def template_meta_name(self):
        """Get the template_meta_name of this StackResources.

        :return: The template_meta_name of this StackResources.
        :rtype: str
        """  # noqa: E501
        return self._template_meta_name

    @template_meta_name.setter
    def template_meta_name(self, template_meta_name):
        """Set the template_meta_name of this StackResources.

        :param template_meta_name: The template_meta_name of this StackResources.
        :type: str
        """  # noqa: E501
        self._template_meta_name = template_meta_name

    @property
    def associations(self):
        """Get the associations of this StackResources.

        :return: The associations of this StackResources.
        :rtype: list[StackAssociations]
        """  # noqa: E501
        return self._associations

    @associations.setter
    def associations(self, associations):
        """Set the associations of this StackResources.

        :param associations: The associations of this StackResources.
        :type: list[StackAssociations]
        """  # noqa: E501
        self._associations = associations

    @property
    def links(self):
        """Get the links of this StackResources.

        :return: The links of this StackResources.
        :rtype: StackLinks
        """  # noqa: E501
        return self._links

    @links.setter
    def links(self, links):
        """Set the links of this StackResources.

        :param links: The links of this StackResources.
        :type: StackLinks
        """  # noqa: E501
        self._links = links

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in self.openapi_types.items():
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, StackResources):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



import asyncio
import logging
import re
import unittest
import os
from datetime import datetime, timezone
from io import StringIO

import pytest
import warnings
from aioresponses import aioresponses

from influxdb_client import Point, WritePrecision, BucketsService, OrganizationsService, Organizations
from influxdb_client.client.exceptions import InfluxDBError
from influxdb_client.client.influxdb_client_async import InfluxDBClientAsync
from influxdb_client.client.query_api import QueryOptions
from influxdb_client.client.warnings import MissingPivotFunction
from influxdb_client.client.write.retry import WritesRetry
from tests.base_test import generate_name


def async_test(coro):
    def wrapper(*args, **kwargs):
        return asyncio.get_event_loop().run_until_complete(coro(*args, **kwargs))

    return wrapper


# noinspection PyMethodMayBeStatic
class InfluxDBClientAsyncTest(unittest.TestCase):

    @async_test
    async def setUp(self) -> None:
        self.client = InfluxDBClientAsync(url="http://localhost:8086", token="my-token", org="my-org")

    @async_test
    async def tearDown(self) -> None:
        if self.client:
            await self.client.close()

    def test_use_async_context_manager(self):
        self.assertIsNotNone(self.client)

    @async_test
    async def test_ping(self):
        ping = await self.client.ping()
        self.assertTrue(ping)

    @async_test
    async def test_version(self):
        version = await self.client.version()
        self.assertTrue(len(version) > 0)

    @async_test
    async def test_build(self):
        build = await self.client.build()
        self.assertEqual('oss', build.lower())

    def test_create_query_api(self):
        query_api = self.client.query_api()
        self.assertIsNotNone(query_api)

    def test_create_write_api(self):
        write_api = self.client.write_api()
        self.assertIsNotNone(write_api)

    def test_create_delete_api(self):
        delete_api = self.client.delete_api()
        self.assertIsNotNone(delete_api)

    @async_test
    async def test_query_tables(self):
        measurement = generate_name("measurement")
        await self._prepare_data(measurement)
        query = f'''
                    from(bucket:"my-bucket") 
                        |> range(start: -10m)
                        |> filter(fn: (r) => r["_measurement"] == "{measurement}")
                '''
        query_api = self.client.query_api()

        tables = await query_api.query(query)
        self.assertEqual(2, len(tables))
        self.assertEqual(1, len(tables[0].records))
        self.assertEqual("New York", tables[0].records[0]['location'])
        self.assertEqual(24.3, tables[0].records[0]['_value'])
        self.assertEqual(1, len(tables[1].records))
        self.assertEqual("Prague", tables[1].records[0]['location'])
        self.assertEqual(25.3, tables[1].records[0]['_value'])

    @async_test
    async def test_query_raw(self):
        measurement = generate_name("measurement")
        await self._prepare_data(measurement)
        query = f'''
                    from(bucket:"my-bucket") 
                        |> range(start: -10m)
                        |> filter(fn: (r) => r["_measurement"] == "{measurement}")
                        |> drop(columns: ["_start", "_stop", "_time", "_measurement"])
                '''
        query_api = self.client.query_api()

        raw = await query_api.query_raw(query)
        self.assertEqual(7, len(raw.splitlines()))
        self.assertEqual(',,0,24.3,temperature,New York', raw.splitlines()[4])
        self.assertEqual(',,1,25.3,temperature,Prague', raw.splitlines()[5])

    @async_test
    async def test_query_stream_records(self):
        measurement = generate_name("measurement")
        await self._prepare_data(measurement)
        query = f'''
                    from(bucket:"my-bucket") 
                        |> range(start: -10m)
                        |> filter(fn: (r) => r["_measurement"] == "{measurement}")
                '''
        query_api = self.client.query_api()

        records = []
        async for record in await query_api.query_stream(query):
            records.append(record)

        self.assertEqual(2, len(records))
        self.assertEqual("New York", records[0]['location'])
        self.assertEqual(24.3, records[0]['_value'])
        self.assertEqual("Prague", records[1]['location'])
        self.assertEqual(25.3, records[1]['_value'])

    @async_test
    async def test_query_data_frame(self):
        measurement = generate_name("measurement")
        await self._prepare_data(measurement)
        query = f'''
                    from(bucket:"my-bucket") 
                        |> range(start: -10m)
                        |> filter(fn: (r) => r["_measurement"] == "{measurement}") 
                        |> group()
                '''
        query_api = self.client.query_api()

        dataframe = await query_api.query_data_frame(query)
        self.assertIsNotNone(dataframe)
        self.assertEqual(2, len(dataframe))
        self.assertEqual(24.3, dataframe['_value'][0])
        self.assertEqual(25.3, dataframe['_value'][1])
        self.assertEqual('temperature', dataframe['_field'][0])
        self.assertEqual('temperature', dataframe['_field'][1])
        self.assertEqual(measurement, dataframe['_measurement'][0])
        self.assertEqual(measurement, dataframe['_measurement'][1])
        self.assertEqual('New York', dataframe['location'][0])
        self.assertEqual('Prague', dataframe['location'][1])

    @async_test
    async def test_query_data_frame_with_warning(self):
        measurement = generate_name("measurement")
        await self._prepare_data(measurement)
        query = f'''
                    from(bucket:"my-bucket") 
                        |> range(start: -10m)
                        |> filter(fn: (r) => r["_measurement"] == "{measurement}") 
                '''
        query_api = self.client.query_api()

        with pytest.warns(MissingPivotFunction) as warnings:
            dataframe = await query_api.query_data_frame(query)
            self.assertIsNotNone(dataframe)
        self.assertEqual(1, len(warnings))

    @async_test
    async def test_query_data_frame_without_warning(self):
        measurement = generate_name("measurement")
        await self._prepare_data(measurement)
        query = f'''
                    from(bucket:"my-bucket") 
                        |> range(start: -10m)
                        |> filter(fn: (r) => r["_measurement"] == "{measurement}") 
                        |> pivot(rowKey:["_time"], columnKey: ["_field"], valueColumn: "_value")
                '''
        query_api = self.client.query_api()

        with warnings.catch_warnings(record=True) as warns:
            dataframe = await query_api.query_data_frame(query)
            self.assertIsNotNone(dataframe)
        self.assertEqual(0, len(warns))

    @async_test
    async def test_write_response_type(self):
        measurement = generate_name("measurement")
        point = Point(measurement).tag("location", "Prague").field("temperature", 25.3)
        response = await self.client.write_api().write(bucket="my-bucket", record=point)

        self.assertEqual(True, response)

    @async_test
    async def test_write_empty_data(self):
        measurement = generate_name("measurement")
        point = Point(measurement).tag("location", "Prague")
        response = await self.client.write_api().write(bucket="my-bucket", record=point)

        self.assertEqual(True, response)

    @async_test
    async def test_write_points_different_precision(self):
        measurement = generate_name("measurement")
        _point1 = Point(measurement).tag("location", "Prague").field("temperature", 25.3) \
            .time(datetime.fromtimestamp(0, tz=timezone.utc), write_precision=WritePrecision.S)
        _point2 = Point(measurement).tag("location", "New York").field("temperature", 24.3) \
            .time(datetime.fromtimestamp(1, tz=timezone.utc), write_precision=WritePrecision.MS)
        _point3 = Point(measurement).tag("location", "Berlin").field("temperature", 24.3) \
            .time(datetime.fromtimestamp(2, tz=timezone.utc), write_precision=WritePrecision.NS)
        await self.client.write_api().write(bucket="my-bucket", record=[_point1, _point2, _point3],
                                            write_precision=WritePrecision.NS)
        query = f'''
                    from(bucket:"my-bucket") 
                        |> range(start: 0)
                        |> filter(fn: (r) => r["_measurement"] == "{measurement}") 
                        |> keep(columns: ["_time"])
                '''
        query_api = self.client.query_api()

        raw = await query_api.query_raw(query)
        self.assertEqual(8, len(raw.splitlines()))
        self.assertEqual(',,0,1970-01-01T00:00:02Z', raw.splitlines()[4])
        self.assertEqual(',,0,1970-01-01T00:00:01Z', raw.splitlines()[5])
        self.assertEqual(',,0,1970-01-01T00:00:00Z', raw.splitlines()[6])

    @async_test
    async def test_delete_api(self):
        measurement = generate_name("measurement")
        await self._prepare_data(measurement)

        successfully = await self.client.delete_api().delete(start=datetime.fromtimestamp(0),
                                                             stop=datetime.now(tz=timezone.utc),
                                                             predicate="location = \"Prague\"", bucket="my-bucket")
        self.assertEqual(True, successfully)
        query = f'''
                        from(bucket:"my-bucket") 
                            |> range(start: -10m)
                            |> filter(fn: (r) => r["_measurement"] == "{measurement}")
                    '''
        query_api = self.client.query_api()
        tables = await query_api.query(query)
        self.assertEqual(1, len(tables))
        self.assertEqual(1, len(tables[0].records))
        self.assertEqual("New York", tables[0].records[0]['location'])
        self.assertEqual(24.3, tables[0].records[0]['_value'])

    @async_test
    async def test_init_from_ini_file(self):
        client_from_config = InfluxDBClientAsync.from_config_file(f'{os.path.dirname(__file__)}/config.ini')
        self.assertEqual("http://localhost:8086", client_from_config.url)
        self.assertEqual("my-org", client_from_config.org)
        self.assertEqual("my-token", client_from_config.token)
        self.assertEqual(6000, client_from_config.api_client.configuration.timeout)
        self.assertEqual(3, len(client_from_config.default_tags))
        self.assertEqual("132-987-655", client_from_config.default_tags["id"])
        self.assertEqual("California Miner", client_from_config.default_tags["customer"])
        self.assertEqual("${env.data_center}", client_from_config.default_tags["data_center"])

        await client_from_config.close()

    @async_test
    async def test_init_from_file_kwargs(self):
        retry = WritesRetry(total=1, retry_interval=2, exponential_base=3)
        client_from_config = InfluxDBClientAsync.from_config_file(f'{os.path.dirname(__file__)}/config.ini',
                                                                  retries=retry)
        self.assertEqual(client_from_config.retries, retry)

        await client_from_config.close()

    @async_test
    async def test_init_from_env(self):
        os.environ["INFLUXDB_V2_URL"] = "http://localhost:8086"
        os.environ["INFLUXDB_V2_ORG"] = "my-org"
        os.environ["INFLUXDB_V2_TOKEN"] = "my-token"
        os.environ["INFLUXDB_V2_TIMEOUT"] = "5500"
        client_from_envs = InfluxDBClientAsync.from_env_properties()
        self.assertEqual("http://localhost:8086", client_from_envs.url)
        self.assertEqual("my-org", client_from_envs.org)
        self.assertEqual("my-token", client_from_envs.token)
        self.assertEqual(5500, client_from_envs.api_client.configuration.timeout)

        await client_from_envs.close()

    @async_test
    async def test_init_from_kwargs(self):
        retry = WritesRetry(total=1, retry_interval=2, exponential_base=3)
        client_from_envs = InfluxDBClientAsync.from_env_properties(retries=retry)

        self.assertEqual(client_from_envs.retries, retry)

        await client_from_envs.close()

    def test_initialize_out_side_async_context(self):
        with pytest.raises(InfluxDBError) as e:
            InfluxDBClientAsync(url="http://localhost:8086", token="my-token", org="my-org")
        self.assertEqual("The async client should be initialised inside async coroutine "
                         "otherwise there can be unexpected behaviour.", e.value.message)

    @async_test
    async def test_username_password_authorization(self):
        await self.client.close()
        self.client = InfluxDBClientAsync(url="http://localhost:8086", username="my-user", password="my-password",
                                          debug=True)
        await self.client.query_api().query("buckets()", "my-org")

    @async_test
    @aioresponses()
    async def test_init_without_token(self, mocked):
        mocked.post('http://localhost/api/v2/query?org=my-org', status=200, body='')
        await self.client.close()
        self.client = InfluxDBClientAsync("http://localhost")
        await self.client.query_api().query("buckets()", "my-org")

    @async_test
    async def test_redacted_auth_header(self):
        await self.client.close()
        self.client = InfluxDBClientAsync(url="http://localhost:8086", token="my-token", org="my-org", debug=True)

        log_stream = StringIO()
        logger = logging.getLogger("influxdb_client.client.http")
        logger.addHandler(logging.StreamHandler(log_stream))

        await self.client.query_api().query("buckets()", "my-org")

        self.assertIn("Authorization: ***", log_stream.getvalue())

    @async_test
    async def test_query_and_debug(self):
        await self.client.close()
        self.client = InfluxDBClientAsync(url="http://localhost:8086", token="my-token", debug=True)
        # Query
        results = await self.client.query_api().query("buckets()", "my-org")
        self.assertIn("my-bucket", list(map(lambda record: record["name"], results[0].records)))
        # Query RAW
        results = await self.client.query_api().query_raw("buckets()", "my-org")
        self.assertIn("my-bucket", results)
        # Bucket API
        buckets_service = BucketsService(api_client=self.client.api_client)
        results = await buckets_service.get_buckets_async()
        self.assertIn("my-bucket", list(map(lambda bucket: bucket.name, results.buckets)))

    @async_test
    @aioresponses()
    async def test_parse_csv_with_new_lines_in_column(self, mocked):
        await self.client.close()
        self.client = InfluxDBClientAsync("http://localhost")
        mocked.post('http://localhost/api/v2/query?org=my-org', status=200, body='''#datatype,string,long,dateTime:RFC3339
#group,false,false,false
#default,_result,,
,result,table,_time
,,0,2022-09-09T10:22:13.744147091Z

#datatype,string,long,string,long,long,long,long,long,long,long,long,long,string,long,long,string
#group,false,false,true,false,false,false,false,false,false,false,false,false,false,false,false,false
#default,_profiler,,,,,,,,,,,,,,,
,result,table,_measurement,TotalDuration,CompileDuration,QueueDuration,PlanDuration,RequeueDuration,ExecuteDuration,Concurrency,MaxAllocated,TotalAllocated,RuntimeErrors,influxdb/scanned-bytes,influxdb/scanned-values,flux/query-plan
,,0,profiler/query,17305459,6292042,116958,0,0,10758125,0,448,0,,0,0,"digraph {
  ""ReadRange4""
  ""keep2""
  ""limit3""

  ""ReadRange4"" -> ""keep2""
  ""keep2"" -> ""limit3""
}

"

#datatype,string,long,string,string,string,long,long,long,long,double
#group,false,false,true,false,false,false,false,false,false,false
#default,_profiler,,,,,,,,,
,result,table,_measurement,Type,Label,Count,MinDuration,MaxDuration,DurationSum,MeanDuration
,,1,profiler/operator,*influxdb.readFilterSource,ReadRange4,1,888209,888209,888209,888209
,,1,profiler/operator,*universe.schemaMutationTransformation,keep2,4,1875,42042,64209,16052.25
,,1,profiler/operator,*universe.limitTransformation,limit3,3,1333,38750,47874,15958''')

        records = []
        await self.client\
            .query_api(QueryOptions(profilers=["operator", "query"],
                                    profiler_callback=lambda record: records.append(record))) \
            .query("buckets()", "my-org")

        self.assertEqual(4, len(records))

    @async_test
    async def test_query_exception_propagation(self):
        await self.client.close()
        self.client = InfluxDBClientAsync(url="http://localhost:8086", token="wrong", org="my-org")

        with pytest.raises(InfluxDBError) as e:
            await self.client.query_api().query("buckets()", "my-org")
        self.assertEqual("unauthorized access", e.value.message)

    @async_test
    async def test_write_exception_propagation(self):
        await self.client.close()
        self.client = InfluxDBClientAsync(url="http://localhost:8086", token="wrong", org="my-org")

        with pytest.raises(InfluxDBError) as e:
            await self.client.write_api().write(bucket="my_bucket",
                                                record="temperature,location=hic cels=")
        self.assertEqual("unauthorized access", e.value.message)
        headers = e.value.headers
        self.assertIsNotNone(headers)
        self.assertIsNotNone(headers.get("Content-Length"))
        self.assertIsNotNone(headers.get("Date"))
        self.assertIsNotNone(headers.get("X-Platform-Error-Code"))
        self.assertIn("application/json", headers.get("Content-Type"))
        self.assertTrue(re.compile("^v.*").match(headers.get("X-Influxdb-Version")))
        self.assertEqual("OSS", headers.get("X-Influxdb-Build"))

    @async_test
    @aioresponses()
    async def test_parse_utf8_two_bytes_character(self, mocked):
        await self.client.close()
        self.client = InfluxDBClientAsync("http://localhost")

        body = '''#group,false,false,false,false,true,true,true
#datatype,string,long,dateTime:RFC3339,string,string,string,string
#default,_result,,,,,,
,result,table,_time,_value,_field,_measurement,type
'''
        for i in range(1000):
            body += f",,0,2022-10-13T12:28:31.{i}Z,ÂÂÂ,value,async,error\n"

        mocked.post('http://localhost/api/v2/query?org=my-org', status=200, body=body)

        data_frame = await self.client.query_api().query_data_frame("from()", "my-org")
        self.assertEqual(1000, len(data_frame))

    @async_test
    async def test_management_apis(self):
        service = OrganizationsService(api_client=self.client.api_client)
        results = await service.get_orgs_async()
        self.assertIsInstance(results, Organizations)
        self.assertIn("my-org", list(map(lambda org: org.name, results.orgs)))

    @async_test
    async def test_trust_env_default(self):
        self.assertFalse(self.client.api_client.rest_client.pool_manager.trust_env)

        await self.client.close()
        self.client = InfluxDBClientAsync(url="http://localhost:8086", token="wrong", org="my-org",
                                          client_session_kwargs={'trust_env': True})
        self.assertTrue(self.client.api_client.rest_client.pool_manager.trust_env)

    async def _prepare_data(self, measurement: str):
        _point1 = Point(measurement).tag("location", "Prague").field("temperature", 25.3)
        _point2 = Point(measurement).tag("location", "New York").field("temperature", 24.3)
        await self.client.write_api().write(bucket="my-bucket", record=[_point1, _point2])



import logging
import unittest
import unittest.mock

import urllib3

from influxdb_client import InfluxLoggingHandler, InfluxDBClient, WriteApi, WritePrecision, Point


class LoggingBaseTestCase(unittest.TestCase):
    fake_line_record = "tag,field=value 123456"
    URL_TOKEN_ORG = {
        'url': 'http://example.com',
        'token': 'my-token',
        'org': 'my-org',
    }
    BUCKET = 'my-bucket'

    def setUp(self) -> None:
        self.mock_InfluxDBClient = unittest.mock.patch("influxdb_client.client.logging_handler.InfluxDBClient").start()
        self.mock_client = unittest.mock.MagicMock(spec=InfluxDBClient)
        self.mock_write_api = unittest.mock.MagicMock(spec=WriteApi)
        self.mock_client.write_api.return_value = self.mock_write_api
        self.mock_InfluxDBClient.return_value = self.mock_client

    def gen_handler_and_logger(self):
        self.handler = InfluxLoggingHandler(**self.URL_TOKEN_ORG, bucket=self.BUCKET)
        self.handler.setLevel(logging.DEBUG)

        self.logger = logging.getLogger("test-logger")
        self.logger.setLevel(logging.DEBUG)

    def tearDown(self) -> None:
        unittest.mock.patch.stopall()


class TestHandlerCreation(LoggingBaseTestCase):
    def test_can_create_handler(self):
        self.handler = InfluxLoggingHandler(**self.URL_TOKEN_ORG, bucket=self.BUCKET)
        self.mock_InfluxDBClient.assert_called_once_with(**self.URL_TOKEN_ORG)
        self.assertEqual(self.BUCKET, self.handler.bucket)
        self.mock_client.write_api.assert_called_once_with()

    def test_can_create_handler_with_optional_args_for_client(self):
        self.handler = InfluxLoggingHandler(**self.URL_TOKEN_ORG, bucket=self.BUCKET,
                                            client_args={'arg2': 2.90, 'optArg': 'whot'})
        self.mock_InfluxDBClient.assert_called_once_with(**self.URL_TOKEN_ORG, arg2=2.90, optArg="whot")
        self.mock_client.write_api.assert_called_once_with()

    def test_can_create_handler_with_args_for_write_api(self):
        self.handler = InfluxLoggingHandler(**self.URL_TOKEN_ORG, bucket=self.BUCKET,
                                            client_args={'arg2': 2.90, 'optArg': 'whot'},
                                            write_api_args={'foo': 'bar'})
        self.mock_InfluxDBClient.assert_called_once_with(**self.URL_TOKEN_ORG, arg2=2.90, optArg="whot")
        self.mock_client.write_api.assert_called_once_with(foo='bar')


class CreatedHandlerTestCase(LoggingBaseTestCase):
    def setUp(self) -> None:
        super().setUp()
        self.gen_handler_and_logger()


class LoggingSetupAndTearDown(CreatedHandlerTestCase):
    def test_is_handler(self):
        self.assertIsInstance(self.handler, logging.Handler)

    def test_set_up_client(self):
        self.mock_InfluxDBClient.assert_called_once()

    def test_closes_connections_on_close(self):
        self.handler.close()
        self.mock_write_api.close.assert_called_once()
        self.mock_client.close.assert_called_once()

    def test_handler_can_be_attached_to_logger(self):
        self.logger.addHandler(self.handler)
        self.assertTrue(self.logger.hasHandlers())
        self.assertTrue(self.handler in self.logger.handlers)


class LoggingWithAttachedHandler(CreatedHandlerTestCase):

    def setUp(self) -> None:
        super().setUp()
        self.logger.addHandler(self.handler)


class LoggingHandlerTest(LoggingWithAttachedHandler):

    def test_can_log_str(self):
        self.logger.debug(self.fake_line_record)
        self.mock_write_api.write.assert_called_once_with(bucket="my-bucket", record=self.fake_line_record)

    def test_can_log_points(self):
        point = Point("measurement").field("field_name", "field_value").time(333, WritePrecision.NS)
        self.logger.debug(point)
        self.mock_write_api.write.assert_called_once_with(bucket="my-bucket", record=str(point))

    def test_catches_urllib_exceptions(self):
        self.mock_write_api.write.side_effect = urllib3.exceptions.HTTPError()
        try:
            with unittest.mock.patch("logging.sys.stderr") as _:
                # Handler writes logging errors to stderr. Don't display it in the test output.
                self.logger.debug(self.fake_line_record)
        finally:
            self.mock_write_api.write.side_effect = None

    def test_raises_on_exit(self):
        try:
            self.mock_write_api.write.side_effect = KeyboardInterrupt()
            self.assertRaises(KeyboardInterrupt, self.logger.debug, self.fake_line_record)
            self.mock_write_api.write.side_effect = SystemExit()
            self.assertRaises(SystemExit, self.logger.debug, self.fake_line_record)
        finally:
            self.mock_write_api.write.side_effect = None

    def test_can_set_bucket(self):
        self.handler.bucket = "new-bucket"
        self.logger.debug(self.fake_line_record)
        self.mock_write_api.write.assert_called_once_with(bucket="new-bucket", record=self.fake_line_record)

    def test_can_pass_bucket_on_log(self):
        self.logger.debug(self.fake_line_record, extra={'bucket': "other-bucket"})
        self.mock_write_api.write.assert_called_once_with(bucket="other-bucket", record=self.fake_line_record)

    def test_can_pass_optional_params_on_log(self):
        self.logger.debug(self.fake_line_record, extra={'org': "other-org", 'write_precision': WritePrecision.S,
                                                        "arg3": 3, "arg2": "two"})
        self.mock_write_api.write.assert_called_once_with(bucket="my-bucket", org='other-org',
                                                          record=self.fake_line_record,
                                                          write_precision=WritePrecision.S,
                                                          arg3=3, arg2="two")

    def test_formatter(self):
        class MyFormatter(logging.Formatter):
            def format(self, record: logging.LogRecord) -> str:
                time_ns = int(record.created * 1e9) * 0 + 123
                return f"{record.name},level={record.levelname} message=\"{record.msg}\" {time_ns}"

        self.handler.setFormatter(MyFormatter())
        msg = "a debug message"
        self.logger.debug(msg)
        expected_record = f"test-logger,level=DEBUG message=\"{msg}\" 123"
        self.mock_write_api.write.assert_called_once_with(bucket="my-bucket", record=expected_record)


if __name__ == "__main__":
    unittest.main()
